@@ -61,16 +61,18 @@ static const struct configdef_kv def_slideshow[] = {
 
 static const struct configdef_kv def_gallery[] = {
     { CFG_GLRY_SIZE,      "200"       },
+    { CFG_GLRY_PADDING,   "5"         },
     { CFG_GLRY_CACHE,     "100"       },
     { CFG_GLRY_PRELOAD,   CFG_NO      },
     { CFG_GLRY_PSTORE,    CFG_NO      },
-    { CFG_GLRY_FILL,      CFG_YES     },
+    { CFG_GLRY_ASPECT,    "fill"      },
     { CFG_GLRY_AA,        "mks13"     },
     { CFG_GLRY_WINDOW,    "#00000000" },
     { CFG_GLRY_BKG,       "#202020ff" },
     { CFG_GLRY_SELECT,    "#404040ff" },
-    { CFG_GLRY_BORDER,    "#000000ff" },
-    { CFG_GLRY_SHADOW,    "#000000ff" },
+    { CFG_GLRY_BRD_COLOR, "#aaaaaaff" },
+    { CFG_GLRY_BRD_WIDTH, "5"         },
+    { CFG_GLRY_SSCALE,    "1.15"      },
 };
 
 static const struct configdef_kv def_list[] = {
@@ -736,6 +738,25 @@ ssize_t config_get_num(const struct config* section, const char* key,
     return num;
 }
 
+float config_get_float(const struct config* section, const char* key,
+                       float min_val, float max_val)
+{
+    float num = 0.0f;
+    const char* value = config_get(section, key);
+
+    if (!str_to_float(value, 0, &num) || num < min_val || num > max_val) {
+        str_to_float(config_get_default(section->name, key), 0, &num);
+        fprintf(stderr,
+                "WARNING: "
+                "Invalid config value \"%s = %s\" in section \"%s\": "
+                "expected float in range %.03f-%.03f, "
+                "the default value %.03f will be used\n",
+                key, value, section->name, min_val, max_val, num);
+    }
+
+    return num;
+}
+
 argb_t config_get_color(const struct config* section, const char* key)
 {
     argb_t color = 0;
