@@ -41,57 +41,65 @@ struct config {
 #define CFG_KEYS_GALLERY   CFG_KEYS "." CFG_GALLERY
 
 // Configuration parameters
-#define CFG_GNRL_MODE      "mode"
-#define CFG_GNRL_SIZE      "size"
-#define CFG_GNRL_POSITION  "position"
-#define CFG_GNRL_OVERLAY   "overlay"
-#define CFG_GNRL_DECOR     "decoration"
-#define CFG_GNRL_SIGUSR1   "sigusr1"
-#define CFG_GNRL_SIGUSR2   "sigusr2"
-#define CFG_GNRL_IPC       "ipc"
-#define CFG_GNRL_APP_ID    "app_id"
-#define CFG_DRM_PATH       "path"
-#define CFG_DRM_CONNECTOR  "connector"
-#define CFG_DRM_MODE       "mode"
-#define CFG_VIEW_WINDOW    "window"
-#define CFG_VIEW_TRANSP    "transparency"
-#define CFG_VIEW_SCALE     "scale"
-#define CFG_VIEW_POSITION  "position"
-#define CFG_VIEW_AA        "antialiasing"
-#define CFG_VIEW_LOOP      "loop"
-#define CFG_VIEW_HISTORY   "history"
-#define CFG_VIEW_PRELOAD   "preload"
-#define CFG_VIEW_SSHOW_TM  "time"
+#define CFG_GNRL_MODE     "mode"
+#define CFG_GNRL_SIZE     "size"
+#define CFG_GNRL_POSITION "position"
+#define CFG_GNRL_OVERLAY  "overlay"
+#define CFG_GNRL_DECOR    "decoration"
+#define CFG_GNRL_SIGUSR1  "sigusr1"
+#define CFG_GNRL_SIGUSR2  "sigusr2"
+#define CFG_GNRL_IPC      "ipc"
+#define CFG_GNRL_APP_ID   "app_id"
+
+#define CFG_DRM_PATH      "path"
+#define CFG_DRM_CONNECTOR "connector"
+#define CFG_DRM_MODE      "mode"
+
+#define CFG_VIEW_WINDOW   "window"
+#define CFG_VIEW_TRANSP   "transparency"
+#define CFG_VIEW_SCALE    "scale"
+#define CFG_VIEW_POSITION "position"
+#define CFG_VIEW_AA       "antialiasing"
+#define CFG_VIEW_LOOP     "loop"
+#define CFG_VIEW_HISTORY  "history"
+#define CFG_VIEW_PRELOAD  "preload"
+#define CFG_VIEW_SSHOW_TM "time"
+
 #define CFG_GLRY_SIZE      "size"
+#define CFG_GLRY_PADDING   "padding"
 #define CFG_GLRY_CACHE     "cache"
 #define CFG_GLRY_PRELOAD   "preload"
 #define CFG_GLRY_PSTORE    "pstore"
-#define CFG_GLRY_FILL      "fill"
+#define CFG_GLRY_ASPECT    "aspect"
 #define CFG_GLRY_AA        "antialiasing"
 #define CFG_GLRY_WINDOW    "window"
 #define CFG_GLRY_BKG       "background"
 #define CFG_GLRY_SELECT    "select"
-#define CFG_GLRY_BORDER    "border"
-#define CFG_GLRY_SHADOW    "shadow"
+#define CFG_GLRY_BRD_COLOR "border_color"
+#define CFG_GLRY_BRD_WIDTH "border_width"
+#define CFG_GLRY_SSCALE    "selected_scale"
+
 #define CFG_LIST_FROMFILE  "from_file"
 #define CFG_LIST_ORDER     "order"
 #define CFG_LIST_REVERSE   "reverse"
 #define CFG_LIST_RECURSIVE "recursive"
 #define CFG_LIST_ALL       "all"
 #define CFG_LIST_FSMON     "fsmon"
-#define CFG_FONT_NAME      "name"
-#define CFG_FONT_SIZE      "size"
-#define CFG_FONT_COLOR     "color"
-#define CFG_FONT_BKG       "background"
-#define CFG_FONT_SHADOW    "shadow"
-#define CFG_INFO_SHOW      "show"
-#define CFG_INFO_PADDING   "padding"
-#define CFG_INFO_ITIMEOUT  "info_timeout"
-#define CFG_INFO_STIMEOUT  "status_timeout"
-#define CFG_INFO_TL        "top_left"
-#define CFG_INFO_TR        "top_right"
-#define CFG_INFO_BL        "bottom_left"
-#define CFG_INFO_BR        "bottom_right"
+
+#define CFG_FONT_NAME   "name"
+#define CFG_FONT_SIZE   "size"
+#define CFG_FONT_COLOR  "color"
+#define CFG_FONT_BKG    "background"
+#define CFG_FONT_SHADOW "shadow"
+
+#define CFG_INFO_SHOW     "show"
+#define CFG_INFO_PADDING  "padding"
+#define CFG_INFO_ITIMEOUT "info_timeout"
+#define CFG_INFO_STIMEOUT "status_timeout"
+#define CFG_INFO_TL       "top_left"
+#define CFG_INFO_TR       "top_right"
+#define CFG_INFO_BL       "bottom_left"
+#define CFG_INFO_BR       "bottom_right"
 
 // Some configuration values
 #define CFG_YES        "yes"
@@ -189,6 +197,16 @@ bool config_get_bool(const struct config* section, const char* key);
  */
 ssize_t config_get_num(const struct config* section, const char* key,
                        ssize_t min_val, ssize_t max_val);
+
+/**
+ * Get config parameter as float value.
+ * @param section config section
+ * @param key property key
+ * @return value or default value if user defined value is invalid
+ */
+float config_get_float(const struct config* section, const char* key,
+                       float min_val, float max_val);
+
 /**
  * Get config parameter as ARGB color value.
  * @param section config section
