@@ -14,19 +14,21 @@
 #include "ui/ui.h"
 
 #include <assert.h>
-#include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 
-// Scale for selected thumbnail
-#define THUMB_SELECTED_SCALE 1.15f
-
 // Limits for thumbnail size
 #define THIMB_SIZE_MIN 50
 #define THIMB_SIZE_MAX 1000
 
+static const char* aspect_names[] = {
+    [aspect_fit] = "fit",
+    [aspect_fill] = "fill",
+    [aspect_keep] = "keep",
+};
+
 /** Gallery context. */
 struct gallery {
     size_t cache; ///< Max number of thumbnails in cache
@@ -35,14 +37,15 @@ struct gallery {
     bool thumb_aa_en;      ///< Enable/disable anti-aliasing mode
     enum aa_mode thumb_aa; ///< Anti-aliasing mode
 
-    bool thumb_fill;   ///< Scale mode (fill/fit)
-    bool thumb_pstore; ///< Use persistent storage for thumbnails
+    enum thumb_aspect aspect; ///< Thumbnail aspect ratio (fit/fill/keep)
+    bool thumb_pstore;        ///< Use persistent storage for thumbnails
 
     argb_t clr_window;     ///< Window background
     argb_t clr_background; ///< Tile background
     argb_t clr_select;     ///< Selected tile background
     argb_t clr_border;     ///< Selected tile border
-    argb_t clr_shadow;     ///< Selected tile shadow
+    size_t border_width;   ///< Selected tile border size
+    double selected_scale; ///< Selected tile scale
 
     struct layout layout; ///< Thumbnail layout
 
@@ -86,9 +89,9 @@ static size_t pstore_path(const char* source, char* path, size_t path_max)
     }
 
     // append postfix
-    postfix_len = snprintf(
-        postfix, sizeof(postfix), ".%04x%d%d", (uint16_t)ctx.layout.thumb_size,
-        ctx.thumb_fill ? 1 : 0, ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest);
+    postfix_len = snprintf(postfix, sizeof(postfix), ".%04x%d%d",
+                           (uint16_t)ctx.layout.thumb_size, ctx.aspect,
+                           ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest);
     if (postfix_len <= 0 || len + postfix_len + 1 >= path_max) {
         return 0;
     }
@@ -212,7 +215,7 @@ static void thumb_load(void* data)
     origin = imglist_find(img->source);
     if (!origin ||                 // removed
         image_thumb_get(origin) || // already loaded
-        image_thumb_create(origin, ctx.layout.thumb_size, ctx.thumb_fill,
+        image_thumb_create(origin, ctx.layout.thumb_size, ctx.aspect,
                            ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest)) {
         imglist_unlock();
         app_redraw();
@@ -226,7 +229,7 @@ static void thumb_load(void* data)
     }
     // load entire image and convert it to thumbnail
     if (!image_thumb_get(img) && image_load(img) == imgload_success) {
-        if (image_thumb_create(img, ctx.layout.thumb_size, ctx.thumb_fill,
+        if (image_thumb_create(img, ctx.layout.thumb_size, ctx.aspect,
                                ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest)) {
             if (ctx.thumb_pstore) {
                 // save to thumbnail to persistent storage
@@ -266,7 +269,7 @@ static void thumb_free(void* data)
     image_free(data, IMGDATA_SELF);
 }
 
-/** Recreate thumnail load queue. */
+/** Recreate thumbnail load queue. */
 static void thumb_requeue(void)
 {
     struct image* queue;
@@ -333,13 +336,13 @@ static bool select_next(enum action_type direction)
     if (rc) {
         thumb_requeue();
     }
-    imglist_unlock();
-
     if (rc) {
         info_reset(ctx.layout.current);
     }
     ui_set_title(ctx.layout.current->name);
     info_update_index(info_index, ctx.layout.current->index, imglist_size());
+    imglist_unlock();
+
     app_redraw();
 
     return rc;
@@ -425,68 +428,86 @@ static void thumb_resize(const char* params)
 static void draw_thumbnail(struct pixmap* window,
                            const struct layout_thumb* lth)
 {
+    const bool selected = (lth == layout_current(&ctx.layout));
     const struct pixmap* pm = image_thumb_get(lth->img);
-    ssize_t x = lth->x;
-    ssize_t y = lth->y;
 
-    if (lth != layout_current(&ctx.layout)) {
-        pixmap_fill(window, x, y, ctx.layout.thumb_size, ctx.layout.thumb_size,
-                    ctx.clr_background);
-        if (pm) {
-            x += ctx.layout.thumb_size / 2 - pm->width / 2;
-            y += ctx.layout.thumb_size / 2 - pm->height / 2;
-            pixmap_copy(pm, window, x, y);
+    // calculate tile position/size
+    ssize_t tile_x = lth->x;
+    ssize_t tile_y = lth->y;
+    size_t tile_w = ctx.layout.thumb_size;
+    size_t tile_h = ctx.layout.thumb_size;
+    if (selected) {
+        tile_w *= ctx.selected_scale;
+        tile_h *= ctx.selected_scale;
+        tile_x -= tile_w / 2 - ctx.layout.thumb_size / 2;
+        tile_y -= tile_h / 2 - ctx.layout.thumb_size / 2;
+
+        // prevent going beyond the window
+        if (tile_x + tile_w + ctx.border_width > window->width) {
+            tile_x = window->width - tile_w - ctx.border_width;
         }
+        if (tile_x < (ssize_t)ctx.border_width) {
+            tile_x = ctx.border_width;
+        }
+        if (tile_y + tile_h + ctx.border_width > window->height) {
+            tile_y = window->height - tile_h - ctx.border_width;
+        }
+        if (tile_y < (ssize_t)ctx.border_width) {
+            tile_y = ctx.border_width;
+        }
+    }
+
+    // calculate thumbnail position/size
+    ssize_t th_x = tile_x;
+    ssize_t th_y = tile_y;
+    size_t th_w = tile_w;
+    size_t th_h = tile_h;
+    if (pm && (ctx.aspect == aspect_keep || ctx.aspect == aspect_fit)) {
+        th_w = pm->width;
+        th_h = pm->height;
+        if (selected) {
+            th_w *= ctx.selected_scale;
+            th_h *= ctx.selected_scale;
+        }
+        th_x += tile_w / 2 - th_w / 2;
+        th_y += tile_h / 2 - th_h / 2;
+    }
+
+    // draw background
+    const argb_t bkg_c = selected ? ctx.clr_select : ctx.clr_background;
+    ssize_t bkg_x, bkg_y;
+    size_t bkg_w, bkg_h;
+    if (ctx.aspect == aspect_keep) {
+        bkg_x = th_x;
+        bkg_y = th_y;
+        bkg_w = th_w;
+        bkg_h = th_h;
     } else {
-        // currently selected item
-        const size_t thumb_size = THUMB_SELECTED_SCALE * ctx.layout.thumb_size;
-        const ssize_t thumb_offset = (thumb_size - ctx.layout.thumb_size) / 2;
+        bkg_x = tile_x;
+        bkg_y = tile_y;
+        bkg_w = tile_w;
+        bkg_h = tile_h;
+    }
+    pixmap_fill(window, bkg_x, bkg_y, bkg_w, bkg_h, bkg_c);
 
-        x = max(0, x - thumb_offset);
-        y = max(0, y - thumb_offset);
-        if (x + thumb_size >= window->width) {
-            x = window->width - thumb_size;
-        }
-
-        pixmap_fill(window, x, y, thumb_size, thumb_size, ctx.clr_select);
-
-        if (pm) {
-            const size_t thumb_w = pm->width * THUMB_SELECTED_SCALE;
-            const size_t thumb_h = pm->height * THUMB_SELECTED_SCALE;
-            const ssize_t tx = x + thumb_size / 2 - thumb_w / 2;
-            const ssize_t ty = y + thumb_size / 2 - thumb_h / 2;
-            software_render(pm, window, tx, ty, THUMB_SELECTED_SCALE,
+    // draw thumbnail
+    if (pm) {
+        if (selected) {
+            software_render(pm, window, th_x, th_y, ctx.selected_scale,
                             ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest, false);
+        } else {
+            pixmap_copy(pm, window, th_x, th_y);
         }
+    }
 
-        // shadow
-        if (ARGB_GET_A(ctx.clr_shadow)) {
-            const argb_t base = ctx.clr_shadow & 0x00ffffff;
-            const uint8_t alpha = ARGB_GET_A(ctx.clr_shadow);
-            const size_t width =
-                max(1, (double)thumb_size / 15.0 * ((double)alpha / 255.0));
-            const size_t alpha_step = alpha / width;
-
-            for (size_t i = 0; i < width; ++i) {
-                const ssize_t lx = i + x + thumb_size;
-                const ssize_t ly = y + width;
-                const size_t lh = thumb_size - (width - i);
-                const argb_t color = base | ARGB_SET_A(alpha - i * alpha_step);
-                pixmap_vline(window, lx, ly, lh, color);
-            }
-            for (size_t i = 0; i < width; ++i) {
-                const ssize_t lx = x + width;
-                const ssize_t ly = y + thumb_size + i;
-                const size_t lw = thumb_size - (width - i) + 1;
-                const argb_t color = base | ARGB_SET_A(alpha - i * alpha_step);
-                pixmap_hline(window, lx, ly, lw, color);
-            }
-        }
-
-        // border
-        if (ARGB_GET_A(ctx.clr_border)) {
-            pixmap_rect(window, x, y, thumb_size, thumb_size, ctx.clr_border);
-        }
+    // draw border
+    if (selected && ARGB_GET_A(ctx.clr_border) && ctx.border_width > 0) {
+        const size_t border_x = bkg_x - ctx.border_width;
+        const size_t border_y = bkg_y - ctx.border_width;
+        const size_t border_w = bkg_w + ctx.border_width * 2;
+        const size_t border_h = bkg_h + ctx.border_width * 2;
+        pixmap_rect(window, border_x, border_y, border_w, border_h,
+                    ctx.border_width, ctx.clr_border);
     }
 }
 
@@ -657,7 +678,7 @@ static void on_activate(struct image* image)
     layout_resize(&ctx.layout, ui_get_width(), ui_get_height());
 
     if (!image_thumb_get(image)) {
-        image_thumb_create(image, ctx.layout.thumb_size, ctx.thumb_fill,
+        image_thumb_create(image, ctx.layout.thumb_size, ctx.aspect,
                            ctx.thumb_aa_en ? ctx.thumb_aa : aa_nearest);
     }
 
@@ -681,7 +702,8 @@ void gallery_init(const struct config* cfg, struct mode* handlers)
     const struct config* section = config_section(cfg, CFG_GALLERY);
 
     const size_t ts = config_get_num(section, CFG_GLRY_SIZE, 1, 4096);
-    layout_init(&ctx.layout, ts);
+    const size_t ps = config_get_num(section, CFG_GLRY_PADDING, 0, 256);
+    layout_init(&ctx.layout, ts, ps);
 
     ctx.cache = config_get_num(section, CFG_GLRY_CACHE, 0, SSIZE_MAX);
     ctx.preload = config_get_bool(section, CFG_GLRY_PRELOAD);
@@ -694,14 +716,17 @@ void gallery_init(const struct config* cfg, struct mode* handlers)
         config_error_val(section->name, CFG_GLRY_AA);
     }
 
-    ctx.thumb_fill = config_get_bool(section, CFG_GLRY_FILL);
+    ctx.aspect = config_get_oneof(section, CFG_GLRY_ASPECT, aspect_names,
+                                  ARRAY_SIZE(aspect_names));
     ctx.thumb_pstore = config_get_bool(section, CFG_GLRY_PSTORE);
 
     ctx.clr_window = config_get_color(section, CFG_GLRY_WINDOW);
     ctx.clr_background = config_get_color(section, CFG_GLRY_BKG);
     ctx.clr_select = config_get_color(section, CFG_GLRY_SELECT);
-    ctx.clr_border = config_get_color(section, CFG_GLRY_BORDER);
-    ctx.clr_shadow = config_get_color(section, CFG_GLRY_SHADOW);
+    ctx.clr_border = config_get_color(section, CFG_GLRY_BRD_COLOR);
+    ctx.border_width = config_get_num(section, CFG_GLRY_BRD_WIDTH, 0, 256);
+    ctx.selected_scale =
+        config_get_float(section, CFG_GLRY_SSCALE, 1.0f, 10.0f);
 
     // load key bindings
     ctx.kb = keybind_load(config_section(cfg, CFG_KEYS_GALLERY));
