@@ -10,6 +10,7 @@
 #include "ui/ui.h"
 
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -116,7 +117,6 @@ static void on_timeout(void* data)
  */
 static void timeout_init(struct timeout* timeout)
 {
-    timeout->fd = -1;
     timeout->active = true;
     timeout->fd = fdtimer_add(on_timeout, timeout);
 }
@@ -312,8 +312,14 @@ static void print_block(struct pixmap* wnd, enum position pos,
         const struct field* field = arr_nth(fields, i);
         const struct keyval* origin = &ctx.fields[field->type];
 
-        if (field->type == info_status && !ctx.status.active) {
-            continue;
+        if (field->type == info_status) {
+            if (!ctx.status.active) {
+                continue;
+            }
+        } else {
+            if (!ctx.info.active) {
+                continue;
+            }
         }
 
         if (field->type == info_exif) {
@@ -517,7 +523,7 @@ void info_init(const struct config* cfg, const char* mode)
         config_get_num(section, CFG_INFO_STIMEOUT, 0, 1024) * 1000;
     timeout_init(&ctx.status);
 
-    info_set_default(mode);
+    info_set_default(mode, 0);
     info_reinit();
 }
 
@@ -555,39 +561,78 @@ void info_destroy(void)
     }
 }
 
-void info_switch(const char* name)
+void info_switch(const char* expression)
 {
-    timeout_reset(&ctx.info);
+    struct str_slice modes[6];
+    size_t modes_num = 0;
 
-    if (name && *name) {
-        if (strcmp(name, "off") == 0) {
+    if (!ctx.info.active && ctx.show) {
+        timeout_reset(&ctx.info);
+        return; // no switch, just show existing scheme hidden by timeout
+    }
+
+    if (expression && *expression) {
+        modes_num = str_split(expression, ',', modes, ARRAY_SIZE(modes));
+    }
+
+    if (modes_num == 0) { // switch to next available mode
+        struct scheme* next = ctx.current->next;
+        ctx.show = !!next;
+        ctx.current = next ? next : ctx.schemes;
+    } else { // switch between specified modes
+        const char* cur_name = ctx.show ? ctx.current->name : "off";
+        const size_t cur_len = strlen(cur_name);
+        size_t mode_idx = SIZE_MAX;
+
+        if (modes_num > ARRAY_SIZE(modes)) {
+            modes_num = ARRAY_SIZE(modes);
+        }
+
+        // find current mode slice
+        for (size_t i = 0; i < modes_num; ++i) {
+            if (modes[i].len == cur_len &&
+                strncmp(modes[i].value, cur_name, cur_len) == 0) {
+                mode_idx = i;
+                break;
+            }
+        }
+
+        // get next mode slice
+        if (mode_idx == SIZE_MAX || ++mode_idx >= modes_num) {
+            mode_idx = 0;
+        }
+
+        // set next mode
+        if (modes[mode_idx].len == 3 &&
+            strncmp(modes[mode_idx].value, "off", 3) == 0) {
             ctx.show = false;
-        } else {
-            info_set_default(name);
+        } else if (info_set_default(modes[mode_idx].value,
+                                    modes[mode_idx].len)) {
             ctx.show = true;
         }
-    } else if (!ctx.show) {
-        ctx.show = true;
-    } else {
-        ctx.current = ctx.current->next;
-        if (!ctx.current) {
-            ctx.show = false;
-            ctx.current = ctx.schemes;
-        }
     }
+
+    timeout_reset(&ctx.info);
 }
 
-void info_set_default(const char* name)
+bool info_set_default(const char* name, size_t len)
 {
     struct scheme* it = ctx.schemes;
-    while (it && strcmp(name, it->name) != 0) {
+
+    if (len == 0) {
+        len = strlen(name);
+    }
+
+    while (it) {
+        if (strlen(it->name) == len && strncmp(name, it->name, len) == 0) {
+            ctx.current = it;
+            return true;
+        }
         it = it->next;
     }
-    if (it) {
-        ctx.current = it;
-    } else {
-        fprintf(stderr, "Invalid info scheme: %s\n", name);
-    }
+
+    fprintf(stderr, "Invalid info scheme: %.*s\n", (int)len, name);
+    return false;
 }
 
 void info_reset(const struct image* img)
